// archive_test.cpp : R\[ AvP[ṼGg |Cg`܂B
//

#include "stdafx.h"
#include "../ArchiveManager/ArchiveManager.h"

#ifdef _DEBUG
#pragma comment(lib, "../Debug/ArchiveManager.lib")
#else
#pragma comment(lib, "../Release/ArchiveManager.lib")
#endif

#define STR_SIZE 256

static const WCHAR *archiveNames[] = {
	L"Unspported",
	L"zip",
	L"lzh",
	L"cab", 
	L"rar",
	L"iso",
};

void usage()
{
	puts("test [l | x | m] [target archive file]\n"
		 "  Listing and Extracting test program"
		 "example:\n"
		 "\ttest l test.zip : List contents of archive\n"
		 "\ttest x test.zip : Extract files from archive(not to create directory)\n"
		 "\ttest m test.zip : Extract to memory(print dump)\n"
		 "\ttest h test.zip : Extract to handle\n"
		 "\ttest o test.zip : List contents | Open with handle\n"
		 );
}

BOOL  ExtractCallback(LPVOID param, UInt64 completed, UInt64 total)
{
	if (completed && completed  % 20 == 0) {
		// Print percentage
		fprintf(stderr, "%I64u/%I64u %I64u %\r", completed , total, completed * 100/ total);

#if 0
		// Stop if over 94%
		if (94 < (completed * 100 / total)) {
			printf("stop by callback!!\n");
			return FALSE;	// Abort extracting
		}
#endif
	}
	return TRUE;
}

void hexdump(const void *p, DWORD size);

void PrintErrorString(const WCHAR *function, DWORD err)
{
	const WCHAR *p;
	switch ( err ) {
		case ERROR_INVALID_DATA:
			p = L"A[JCũf[^sł";
			break;
		case ERROR_CRC:
			p = L"CRC̒lv܂";
			break;
		case ERROR_UNSUPPORTED_TYPE:
			p = L"T|[gȂނ̃A[JCuEA[JCũt@C";
			break;
		case ERROR_OUTOFMEMORY:
			p = L"s邽߂̃܂";
			break;
		case ERROR_INTERNAL_ERROR:
			p = L"G[";
			break;
		default:
			_tprintf( L"Error: %s %u\n", function, err );
			return;
	}
	_tprintf( L"Error: %s %s\n", function, p );
}

int _tmain(int argc, _TCHAR* argv[])
{
	HARCHIVE hArchive;
	UInt32 num, i;
	DWORD dwVersion;
	HANDLE hFile;

	_wsetlocale( LC_ALL, L"" );

	if (argc != 3) {
		usage();
		return -1;
	}

	_TCHAR command = *argv[1];
	if (command != L'l' && command != L'x' && command != L'm' && command != L'h' && command != L'o') {
		usage();
		return -1;
	}

	// Check ArchiveManager's version
	if ((dwVersion = GetAMVersion()) != ARCHIVE_MANAGER_VERSION) {
		_tprintf(L"ArchiveManager.dll is old version\n");
	} else {
		dwVersion = ARCHIVE_MANAGER_VERSION;
	}
	// Print ArchiveManager.dll version
	_tprintf(L"== test powerd by ArchiveManager.dll ver %d.%d", HIWORD(dwVersion), LOWORD(dwVersion));

	// Print 7z.dll version
	WCHAR ver7z[STR_SIZE];
	if (Get7zVersion(ver7z, STR_SIZE)) {
		_tprintf(L", 7z.dll ver %s", ver7z);
	}
	_tprintf(L" ==\n\n");

	// Open archive
	if (command == L'o') {
		// with handle
		hFile = CreateFile(argv[2], GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_FLAG_DELETE_ON_CLOSE, NULL);
		if (hFile == INVALID_HANDLE_VALUE) {
			_tprintf(L"Failed: CreateFile %s\n", argv[2]);
			return -1;
		}
		hArchive = OpenArchiveFromHandle(hFile);
		_tprintf(L"Open with Handle(FILE_FLAG_DELETE_ON_CLOSE)\n");
	} else {
		hArchive = OpenArchive(argv[2]);
	}
	if (hArchive == NULL) {
		_tprintf(L"Failed open file: %d\n", GetAMLastError());
		return -1;
	}

	// Print archive type
	_tprintf(L"Opened: \"%s\" as [%s]\n", argv[2], archiveNames[GetArchiveType(hArchive)]);

	// Print number of files
	num = GetFileCount(hArchive);
	_tprintf(L"File Count: %d\n", num);

	// Flag
	bool fileBad = false;

	// Follow files of archive
	for (i = 0; i < num; i++) {
		// Get file path
		WCHAR path[STR_SIZE] = { 0 };
		DWORD strPathSize = GetFilePathFromArchive(hArchive, i, path, STR_SIZE);

		// Get file name
		WCHAR name[STR_SIZE] = { 0 };
		DWORD strSize = GetFileNameFromArchive(hArchive, i, name, STR_SIZE);
		UInt64 size = GetFileSizeFromArchive(hArchive, i);

		// Check size
		if (STR_SIZE < strSize) {
			_tprintf(L"Skipped[%d]: File name is too long\n", i);
			continue;
		}

		// Print file path
		_tprintf(L"%13I64u | ", size);
		_tprintf(L"%Path: %s\n", path);
		_tprintf(L"\t\tName: %s(strSize: %d)\n", name, strSize);

		// Extract to File
		if (command == L'x') {
			BOOL res = ExtractToFile(hArchive, i, name, ExtractCallback, NULL);
			fprintf(stderr, "\n");

			if (res) {
				_tprintf(L"OK\n");
			} else {
				PrintErrorString( L"ExtractToFile", GetAMLastError() );
				fileBad = true;
			}
		}
		// Extract to Memory
		else if (command == L'm') {
			if (size <= INT_MAX) {
				BYTE *mem = new BYTE[size];
				BOOL res = ExtractToMemory(hArchive, i, mem, size, ExtractCallback, NULL);
				if (res) {
					hexdump(mem, (DWORD)size);
				} else {
					PrintErrorString( L"ExtractToMemory", GetAMLastError() );
					fileBad = true;
				}
				delete [] mem;
			} else {
				_tprintf(L"File size is too big for extracting to memory\n");
			}

		// Extract to Handle
		} else if (command == L'h') {
			HANDLE hFile = CreateFile(name, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS , FILE_FLAG_DELETE_ON_CLOSE, NULL);
			if (hFile == INVALID_HANDLE_VALUE) {
				_tprintf(L"Fialed to CreateFile\n");
			} else {
				BOOL res = ExtractToHandle(hArchive, i, hFile, ExtractCallback, NULL);
				if ( res ) {
					CloseHandle(hFile);
				} else {
					PrintErrorString( L"ExtractToHandle", GetAMLastError() );
					fileBad = true;
				}
			}
		}

	}

	// Close handle
	CloseArchive(hArchive);

	return fileBad ? -1 : 0;
}


void hexdump(const void *p, DWORD size)
{
	for (DWORD i = 0; i < size; i++) {
		for (int j = 0; j < 20 && i < size; i++, j++) {
			_tprintf(L"%2x ", *(((BYTE*)p + i)));
		}
		_tprintf(L"\n");
	}
}
